/* GStreamer
 * Copyright (C) 1999,2000,2001,2002 Erik Walthinsen <omega@cse.ogi.edu>
 *                    2000,2001,2002 Wim Taymans <wtay@chello.be>
 *                              2002 Steve Baker <steve@stevebaker.org>
 *								2003 Julien Moutte <julien@moutte.net>
 *
 * gstmediaplay.c: Media Player widget for Gst-Player
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <locale.h>
#include <config.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gnome.h>
#include <gst/gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>

#include "gstmediaplay.h"
#include "gstvideowidget.h"
#include "gstpreferences.h"
#include "gstmediainfo.h"
#include "gstcontrol.h"
#include "gstplayer-marshal.h"
#include "gtk-playlist.h"

static GtkVBoxClass *parent_class = NULL;

enum {
	STATE_CHANGE,
	DISPLAY_MODE_CHANGE,
	CURRENT_LOCATION,
	LAST_SIGNAL
};

static gint gst_media_play_signals [LAST_SIGNAL] = { 0 };


/* ============================================================= */
/*                                                               */
/*                       Private Methods                         */
/*                                                               */
/* ============================================================= */

/* =========================================== */
/*                                             */
/*                  Tool Box                   */
/*                                             */
/* =========================================== */

static gchar*
gst_media_play_get_ui_file (const gchar *fname)
{
	gchar *full_fname;
	
	/* Searching for glade file in uninstalled dir */
	
	full_fname = g_strjoin("/", GLADEUI_UNINSTALLED_DIR, fname, NULL);
	if ( g_file_test(full_fname, G_FILE_TEST_IS_REGULAR) )
		return full_fname;
	
	if (full_fname) g_free(full_fname);
		
	/* Searching for glade file in installed dir */
	
	full_fname = g_strjoin("/", GLADEUI_DIR, fname, NULL);
	if ( g_file_test(full_fname, G_FILE_TEST_IS_REGULAR) )
		return full_fname;
	
	/* Not found */
	
	if (full_fname) g_free(full_fname);
		
	return NULL;
}

static GladeXML*
gst_media_play_get_glade_xml_from_file (	const gchar *fname,
											const gchar *root,
											const gchar *domain)
{
	return glade_xml_new(fname, root, domain);
}

static GladeXML*
gst_media_play_get_glade_xml (	const gchar *fname,
								const gchar *root,
								const gchar *domain)
{
	gchar *full_fname = NULL;
	GladeXML *xml = NULL;
	full_fname = gst_media_play_get_ui_file (fname);
	g_return_val_if_fail(full_fname != NULL, NULL);
	xml = gst_media_play_get_glade_xml_from_file (full_fname, root, domain);
	if (full_fname) g_free (full_fname);
	return xml;
}

static void
gst_media_error_dialog (const gchar *message)
{
	GtkWidget *dialog;
	/* throw up a dialog box; FIXME: we don't have a parent */
	/* FIXME: maybe even fallback the GError and do error
	* handling higher up ? */
	dialog = gtk_message_dialog_new (NULL, 0,
			                 GTK_MESSAGE_ERROR,
					 GTK_BUTTONS_CLOSE,
					 message);
	gtk_dialog_run (GTK_DIALOG (dialog));
	/*
	gtk_widget_show (dialog);
	g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (dialog));
				  */
}

/* =========================================== */
/*                                             */
/*                  GFX stuff                  */
/*                                             */
/* =========================================== */

static void
gst_media_play_toggle_playlist (	GtkWidget *widget,
									GstMediaPlay *mplay)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	if (GTK_WIDGET_VISIBLE(mplay->playlist))
		gtk_widget_hide(mplay->playlist);
	else
		gtk_widget_show (mplay->playlist);
}


static gboolean
animate_control_window (GstMediaPlay *mplay)
{
	static gint height_offset = 0;
	gboolean anim_done = FALSE;
	g_return_val_if_fail(mplay != NULL, FALSE);
	g_return_val_if_fail(GST_IS_MEDIA_PLAY(mplay), FALSE);
	
	if (!mplay->fs_control_bar_visible) {
		height_offset = height_offset + 2;
	}
	else {
		height_offset = height_offset - 2;
	}
	
	if (height_offset >=  mplay->control->allocation.height) {
		height_offset = mplay->control->allocation.height;
		anim_done = TRUE;
	}
	
	if (height_offset <= 0) {
		height_offset = 0;
		anim_done = TRUE;
	}
	
	gtk_window_move (	GTK_WINDOW(mplay->control_fs_window),
						0,
						0 - height_offset);
	
	/* We finished our animation */
	
	if (anim_done) {
		mplay->move_id = 0;
		return FALSE;
	}
	else
		return TRUE;
}

static gboolean
gst_media_play_hide_mouse_cursor (GstMediaPlay *mplay)
{
	g_return_val_if_fail(mplay != NULL, FALSE);
	g_return_val_if_fail(GST_IS_MEDIA_PLAY(mplay), FALSE);
	
	gst_video_widget_set_cursor_visible (
							GST_VIDEO_WIDGET(mplay->video_widget), FALSE);
	
	gtk_widget_hide (GTK_WIDGET(mplay->control_fs_window));
	return TRUE;
}

static void
gst_media_play_set_fullscreen (	GstMediaPlay *mplay,
								gboolean fullscreen,
								gint width, gint height)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
  
	if (fullscreen) {
		
		if (GTK_WIDGET_VISIBLE(mplay->playlist))
			gtk_widget_hide (mplay->playlist);
		
		gtk_widget_hide (GTK_WIDGET(mplay->control));
		
		if (!mplay->control_fs_vbox)
			mplay->control_fs_vbox = GTK_VBOX(gtk_vbox_new ( TRUE, 0));

		if (!mplay->control_fs_window) {
			mplay->control_fs_window = GTK_WINDOW (
										gtk_window_new (GTK_WINDOW_POPUP));
			gtk_container_add (	GTK_CONTAINER(mplay->control_fs_window),
								GTK_WIDGET(mplay->control_fs_vbox));
		}
		
		/* Refcounting our control widget as we don't want to loose it */
		
		g_object_ref (mplay->control);
		
		gtk_container_remove (	GTK_CONTAINER(mplay),
								mplay->control);
		gtk_box_pack_start (	GTK_BOX(mplay->control_fs_vbox),
								mplay->control,
								FALSE, FALSE, 0);
		
		g_object_unref (mplay->control);
		
		gtk_widget_show (GTK_WIDGET(mplay->control));
		gtk_widget_show (GTK_WIDGET(mplay->control_fs_vbox));
		gtk_widget_show (GTK_WIDGET(mplay->control_fs_window));
		
		gtk_window_move (	mplay->control_fs_window,
							0 , 0);
		gtk_window_resize (	mplay->control_fs_window,
							width,
							mplay->control->allocation.height);
		
		gst_control_set_display_mode (	GST_CONTROL(mplay->control),
										GST_MEDIA_PLAY_FULLSCREEN);
		
		gst_video_widget_set_cursor_visible (
							GST_VIDEO_WIDGET(mplay->video_widget), FALSE);
							
		mplay->fs_control_bar_visible = FALSE;
				
		gtk_widget_hide (GTK_WIDGET(mplay->control_fs_window));
	}
	else {
		
		gtk_widget_hide (GTK_WIDGET(mplay->control_fs_window));
		
		/* Refcounting our control widget as we don't want to loose it */
		
		g_object_ref (mplay->control);
		
		gtk_container_remove (	GTK_CONTAINER(mplay->control_fs_vbox),
								mplay->control);
		gtk_box_pack_start (	GTK_BOX(mplay),
								mplay->control,
								FALSE, FALSE, 0);
		
		g_object_unref (mplay->control);
		
		if (mplay->move_id) {
			g_source_remove (mplay->move_id);
			mplay->move_id = 0;
		}
		
		gst_control_set_display_mode (	GST_CONTROL(mplay->control),
										GST_MEDIA_PLAY_NORMAL);
		
		gtk_widget_show (GTK_WIDGET(mplay->control));
	}
}

/* =========================================== */
/*                                             */
/*          Event Handlers, Callbacks          */
/*                                             */
/* =========================================== */

static void
gst_media_play_playlist_changed (	GtkWidget *widget,
									GstMediaPlay *mplay)
{
	gchar *mrl = NULL;
	
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	mrl = gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist));

	if (	mplay->location &&
			mrl &&
			g_strcasecmp (mplay->location, mrl) != 0
		)
	{
		if (gst_media_play_get_state (mplay) == GST_STATE_PLAYING) {
			gst_media_play_set_state (mplay, GST_STATE_READY);
			gst_media_play_set_location (mplay, mrl);
			gst_media_play_set_state (mplay, GST_STATE_PLAYING);
		}
		else
			gst_media_play_set_location (mplay, mrl);
	}
	
	if (mrl) g_free (mrl);

}

static gint
gst_media_play_motion_notify_callback (	GtkWidget *widget,
										GdkEventMotion *event,
										GstMediaPlay *mplay)
{
	gint pointer_x, pointer_y;
	GdkModifierType state;
	
	if (mplay->hide_cursor_id)
		g_source_remove (mplay->hide_cursor_id);
	
	gst_video_widget_set_cursor_visible (GST_VIDEO_WIDGET(widget), TRUE);
	
	if (mplay->display_mode == GST_MEDIA_PLAY_FULLSCREEN) {
		gtk_widget_show (GTK_WIDGET(mplay->control_fs_window));
	}
	
	gdk_window_get_pointer (widget->window, &pointer_x, &pointer_y, &state);
	
	if (mplay->display_mode == GST_MEDIA_PLAY_FULLSCREEN)
		mplay->hide_cursor_id = g_timeout_add (
								3000,
								(GSourceFunc) gst_media_play_hide_mouse_cursor,
								mplay);
	
	return TRUE;
}

static void
gst_media_play_have_video_size (	GstPlay *play,
									gint width, gint height,
									GstMediaPlay *mplay)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	if (mplay->video_widget)
		gst_video_widget_set_source_size(
								GST_VIDEO_WIDGET(mplay->video_widget),
								width, height);
}

static void
gst_media_play_have_xid (	GstPlay *play,
							gint xid,
							GstMediaPlay *mplay)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	if (mplay->video_widget)
		gst_video_widget_set_xembed_xid(
								GST_VIDEO_WIDGET(mplay->video_widget),
								xid);
}

static void
gst_media_play_got_length (	GstPlay *play,
							gint64 length_nanos,
							GstMediaPlay *mplay)
{
	gint seconds;

	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	seconds = (gint) (length_nanos / GST_SECOND);
	mplay->length_nanos = length_nanos;
	gst_control_set_enable_seek (GST_CONTROL (mplay->control), TRUE);
	gst_control_set_length (GST_CONTROL (mplay->control), length_nanos);
}

static void
gst_media_play_time_tick (	GstPlay *play,
							gint64 time_nanos,
							GstMediaPlay *mplay)
{
	gdouble seek_pos;
	gint seconds;

	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	seek_pos = ((gdouble) time_nanos) / ((gdouble) mplay->length_nanos);
	seconds = (gint) (time_nanos / GST_SECOND);
	gst_control_set_seek_pos (mplay->control, seek_pos);
	gst_control_set_time (GST_CONTROL (mplay->control), time_nanos);
}

static void
gst_media_play_volume_changed (	GtkWidget *widget,
								gdouble value,
								GstMediaPlay *mplay)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_play_set_volume (mplay->play, value);
}

static void
gst_media_play_display_mode_changed (	GtkWidget *widget,
										GstMediaPlay *mplay)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	g_signal_emit (	mplay,
					gst_media_play_signals [DISPLAY_MODE_CHANGE],
					0, mplay->display_mode);
}

static void
gst_media_play_seek_to_pos (	GtkWidget *widget,
								gdouble value,
								GstMediaPlay *mplay)
{
	gdouble length_nanos;
	gint64 seek_time;

	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	length_nanos = (gdouble)mplay->length_nanos;
	seek_time = (gint64)(length_nanos * value);
	gst_play_seek_to_time (mplay->play, seek_time);
}

static void
gst_media_play_state_change (	GstPlay *play,
								GstElementState old_state,
								GstElementState new_state,
								GstMediaPlay *mplay)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	if (new_state == GST_STATE_PLAYING)
		gtk_playlist_set_playing (GTK_PLAYLIST(mplay->playlist), TRUE);
	else
		gtk_playlist_set_playing (GTK_PLAYLIST(mplay->playlist), FALSE);
	gst_control_set_state (GST_CONTROL (mplay->control), old_state, new_state);
}

static void
gst_media_play_stream_end (	GstPlay *play,
							GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	gst_video_widget_set_logo_focus (
				GST_VIDEO_WIDGET(mplay->video_widget), TRUE);
	
	gst_media_play_set_state (mplay, GST_STATE_READY);
	
	if (	gtk_playlist_has_next_mrl (GTK_PLAYLIST(mplay->playlist)) ||
			gtk_playlist_get_repeat (GTK_PLAYLIST(mplay->playlist))) {
		gtk_playlist_set_next (GTK_PLAYLIST(mplay->playlist));
		gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
	}
	else {
		gtk_playlist_set_at_start (GTK_PLAYLIST(mplay->playlist));
		gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
	}
}

/* =========================================== */
/*                                             */
/*              Init & Class init              */
/*                                             */
/* =========================================== */

static void
gst_media_play_destroy (GtkObject *object)
{
	GstMediaPlay *mplay;
	
	g_return_if_fail(object != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(object));
	
	mplay = GST_MEDIA_PLAY (object);
	
	if (mplay->location)
		gst_media_play_set_state (mplay, GST_STATE_READY);
	
	if (mplay->hide_cursor_id) {
		g_source_remove (mplay->hide_cursor_id);
		mplay->hide_cursor_id = 0;
	}

	if (mplay->logo_pixbuf) {
		g_object_unref (mplay->logo_pixbuf);
		mplay->logo_pixbuf = NULL;
	}

	if (GTK_IS_WIDGET(mplay->video_widget)) {
		gtk_widget_destroy (mplay->video_widget);
		mplay->video_widget = NULL;
	}

	if (GTK_IS_WIDGET(mplay->playlist)) {
		gtk_widget_destroy (mplay->playlist);
		mplay->playlist = NULL;
	}

	if (GTK_IS_WIDGET(mplay->control)) {
		gtk_widget_destroy (mplay->control);
		mplay->control = NULL;
	}

	if (mplay->play) {
		g_object_unref (mplay->play);
		mplay->play = NULL;
	}
	
	if (mplay->move_id)
		g_source_remove (mplay->move_id);
	if (mplay->hide_cursor_id)
		g_source_remove (mplay->hide_cursor_id);
	
	/* Chain up */
	if (GTK_OBJECT_CLASS(parent_class)->destroy)
		(* GTK_OBJECT_CLASS(parent_class)->destroy) (object);
}

static void
gst_media_play_class_init (GstMediaPlayClass *klass)
{
	GtkWidgetClass  *widget_class = (GtkWidgetClass *) klass;
	GtkObjectClass  *object_class = (GtkObjectClass *) klass;
	GObjectClass    *gobject_class = (GObjectClass *) klass;

	object_class->destroy = gst_media_play_destroy;

	parent_class = gtk_type_class (gtk_vbox_get_type ());

	gst_media_play_signals [STATE_CHANGE] = 
		g_signal_new ("state_change", 
			      G_TYPE_FROM_CLASS (klass), 
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (GstMediaPlayClass, state_changed), 
			      NULL, NULL,
			      gst_marshal_VOID__INT_INT,
			      G_TYPE_NONE, 2, 
			      G_TYPE_INT, G_TYPE_INT);
	gst_media_play_signals [DISPLAY_MODE_CHANGE] =
		g_signal_new ("display_mode_change",
			      G_TYPE_FROM_CLASS (gobject_class),
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (GstMediaPlayClass, display_mode_change),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__INT,
			      G_TYPE_NONE, 1,
			      G_TYPE_INT);
	gst_media_play_signals [CURRENT_LOCATION] =
		g_signal_new ("current_location", 
			      G_TYPE_FROM_CLASS (klass), 
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (GstMediaPlayClass, current_location), 
			      NULL, NULL,
			      gst_play_marshal_VOID__STRING_STRING,
			      G_TYPE_NONE, 2, 
			      G_TYPE_STRING, G_TYPE_STRING);
}

static void
gst_media_play_init (GstMediaPlay *mplay)
{
	GstElement *audio_sink, *video_sink;
	GError *error = NULL;
	gchar *logo_path = NULL;
	
	/* play object */
	mplay->play = gst_play_new (GST_PLAY_PIPE_VIDEO, &error);
	if (error != NULL)
	{
		gst_media_error_dialog (error->message);
		g_error_free (error);
	}
	
	g_return_if_fail(mplay->play != NULL);
	
	audio_sink = gst_gconf_get_default_audio_sink ();

	if (!GST_IS_ELEMENT (audio_sink))
	{
		gst_media_error_dialog (
			"Could not render default GStreamer audio output sink "
			"from GConf /system/gstreamer/default/audiosink key. "
		        "Check if it is set correctly."	
		);
		return;
	}
	gst_play_set_audio_sink (mplay->play, audio_sink);

	video_sink = gst_gconf_get_default_video_sink ();

	if (!GST_IS_ELEMENT (video_sink))
	{
		gst_media_error_dialog (
			"Could not render default GStreamer video output sink "
			"from GConf /system/gstreamer/default/videosink key. "
		        "Check if it is set correctly."	
		);
		return;
	}
	gst_play_set_video_sink (mplay->play, video_sink);

	g_signal_connect (G_OBJECT (mplay->play), "stream_end",
			  G_CALLBACK (gst_media_play_stream_end), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "time_tick", 
			  G_CALLBACK (gst_media_play_time_tick), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "stream_length", 
			  G_CALLBACK (gst_media_play_got_length), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "have_xid", 
			  G_CALLBACK (gst_media_play_have_xid), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "have_video_size", 
			  G_CALLBACK (gst_media_play_have_video_size), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "state_change", 
			  G_CALLBACK (gst_media_play_state_change), mplay);
	
	gtk_box_set_homogeneous (GTK_BOX(mplay), FALSE);
	gtk_box_set_spacing (GTK_BOX(mplay), 0);
	/* player widgets */
	
	mplay->control = gst_control_new (
		gst_media_play_get_glade_xml("mediacontrol.glade", "hbox_controller", NULL),
		gst_media_play_get_glade_xml("mediacontrol.glade", "window_volume_popup", NULL)
	);
	g_return_if_fail(mplay->control != NULL);
	gtk_box_pack_start (GTK_BOX (mplay), GTK_WIDGET (mplay->control), FALSE, FALSE, 0);
	
	mplay->control_fs_window = NULL;
	mplay->control_fs_vbox = NULL;
	
	mplay->video_widget = gst_video_widget_new ();
	g_return_if_fail(mplay->video_widget != NULL);
	
	logo_path = gst_media_play_get_ui_file("gst-player-logo.png");
	
	mplay->logo_pixbuf = gdk_pixbuf_new_from_file (logo_path, NULL);
	if (logo_path) {
		g_free(logo_path);
		logo_path = NULL;
	}
		
	gst_video_widget_set_logo (	GST_VIDEO_WIDGET(mplay->video_widget),
								mplay->logo_pixbuf);
	
	/* Packing the video widget at the end of our Mplay */
	
	gtk_box_pack_end (GTK_BOX (mplay), mplay->video_widget, TRUE, TRUE, 0);
		
	/*gtk_widget_set_size_request (mplay->video_widget, 300, 300);*/

	logo_path = gst_media_play_get_ui_file("gst-player-playing.png");
	
	mplay->play_pixbuf = gdk_pixbuf_new_from_file (logo_path, NULL);
	if (logo_path) {
		g_free(logo_path);
		logo_path = NULL;
	}
		
	mplay->playlist = gtk_playlist_new (
						gst_media_play_get_ui_file("playlist.glade"),
						mplay->play_pixbuf);
	
	g_return_if_fail(mplay->playlist != NULL);
	
	/* Playlist signals */
	
	g_signal_connect (G_OBJECT (mplay->playlist), "delete-event",
			G_CALLBACK (gtk_widget_hide_on_delete), NULL);
	
	g_signal_connect (G_OBJECT (mplay->playlist), "changed",
			G_CALLBACK (gst_media_play_playlist_changed), mplay);
	
	/* VideoWidget signals */

	g_signal_connect (G_OBJECT (mplay->video_widget), "motion-notify-event",
			G_CALLBACK (gst_media_play_motion_notify_callback), mplay);
	
	/* Control signals */

	g_signal_connect (G_OBJECT (mplay->control), "fullscreen_toggled",
			  G_CALLBACK (gst_media_play_display_mode_changed), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "toggle_play",
			  G_CALLBACK (gst_media_play_toggle_play_pause), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "toggle_playlist",
			  G_CALLBACK (gst_media_play_toggle_playlist), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "clicked_next",
			  G_CALLBACK (gst_media_play_next), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "clicked_previous",
			  G_CALLBACK (gst_media_play_previous), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "stop",
			  G_CALLBACK (gst_media_play_stop), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "volume_change",
			  G_CALLBACK (gst_media_play_volume_changed), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "seek_change",
			  G_CALLBACK (gst_media_play_seek_to_pos), mplay);
	
	mplay->display_mode = GST_MEDIA_PLAY_NORMAL;

	gtk_widget_show (mplay->control);
	gtk_widget_show (mplay->video_widget);
	mplay->location = NULL;
}

/* ============================================================= */
/*                                                               */
/*                       Public Methods                          */
/*                                                               */
/* ============================================================= */

/* =========================================== */
/*                                             */
/*   Play/Pause, Stop, Next, Previous, Mute    */
/*                                             */
/* =========================================== */

void
gst_media_play_toggle_play_pause (	GtkWidget *widget,
									GstMediaPlay *mplay)
{
	GstElementState playstate;
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	playstate = gst_play_get_state (mplay->play);

	if (playstate == GST_STATE_PLAYING)
		gst_media_play_set_state (mplay, GST_STATE_PAUSED);
	else if (playstate == GST_STATE_PAUSED)
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
	else if (playstate == GST_STATE_READY) {
		gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
		
	}
}

void
gst_media_play_stop (	GtkWidget *widget,
						GstMediaPlay *mplay)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_state (mplay, GST_STATE_READY);
}

void
gst_media_play_next (	GtkWidget *widget,
						GstMediaPlay *mplay)
{
	g_return_if_fail (mplay != NULL);
	g_return_if_fail (GST_IS_MEDIA_PLAY(mplay));

	if (gtk_playlist_has_next_mrl (GTK_PLAYLIST(mplay->playlist)) ||
		gtk_playlist_get_repeat (GTK_PLAYLIST(mplay->playlist)) ) {
		gst_video_widget_set_logo_focus (
				GST_VIDEO_WIDGET(mplay->video_widget), TRUE);
		
		gst_media_play_set_state (mplay, GST_STATE_READY);
		
		gtk_playlist_set_next (GTK_PLAYLIST(mplay->playlist));
		gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
		
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
	}
}

void
gst_media_play_previous (	GtkWidget *widget,
							GstMediaPlay *mplay)
{
	g_return_if_fail (mplay != NULL);
	g_return_if_fail (GST_IS_MEDIA_PLAY(mplay));
	
	if (gtk_playlist_has_previous_mrl (GTK_PLAYLIST(mplay->playlist)) ||
		gtk_playlist_get_repeat (GTK_PLAYLIST(mplay->playlist)) ) {
		gst_video_widget_set_logo_focus (
				GST_VIDEO_WIDGET(mplay->video_widget), TRUE);
		
		gst_media_play_set_state (mplay, GST_STATE_READY);
		
		gtk_playlist_set_previous (GTK_PLAYLIST(mplay->playlist));
		gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
		
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
	}
}

void
gst_media_play_toggle_mute (	GtkWidget *widget,
								GstMediaPlay *mplay)
{
	gboolean muted;
	
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	muted = !gst_play_get_mute(mplay->play);
	gst_play_set_mute(mplay->play, muted);
	gst_control_set_mute(GST_CONTROL(mplay->control), muted);
}

/* =========================================== */
/*                                             */
/*                  GFX stuff                  */
/*                                             */
/* =========================================== */

void
gst_media_play_show_about (GstMediaPlay *mplay)
{
	static GtkWidget *about;

	const gchar *authors[] = {
		"Steve Baker <stevebaker_org@yahoo.co.uk>",
		"Richard Boulton <richard@tartarus.org>",
		"Ronald Bultje <rbultje@ronald.bitfreak.net>",
		"Arik Devens <arik@gnome.org>",
		"Chris Emerson (PPC port)",
		"Benjamin Otte <in7y118@public.uni-hamburg.de>",
		"Wim Taymans <wim.taymans@tvd.be>",
		"Erik Walthinsen <omega@cse.ogi.edu>", 
		"Thomas Vander Stichele <thomas@apestaart.org>",
		"Julien Moutte <julien@moutte.net>",
		NULL};

	if (about != NULL) {
		gdk_window_raise (about->window);
		gdk_window_show (about->window);
		return;
	}

	about = gnome_about_new (
				"GStreamer Player",
				GST_PLAYER_VERSION, 
				"(C) 1999-2003 The GStreamer Team",
				"A generic media player for the GStreamer media framework.",
				(const char **)authors,
				NULL,
				NULL,
				NULL);

	g_signal_connect (	G_OBJECT (about), "destroy",
						G_CALLBACK (gtk_widget_destroyed), &about);
	
	gtk_widget_show (about);
}

void
gst_media_play_set_display_mode (	GstMediaPlay *mplay,
									GstMediaPlayMode display_mode,
									gint width, gint height)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	gst_control_set_display_mode(GST_CONTROL(mplay->control), display_mode);
	
	switch (display_mode){
		case GST_MEDIA_PLAY_FULLSCREEN:
			if (mplay->display_mode == GST_MEDIA_PLAY_NORMAL)
				gst_media_play_set_fullscreen (mplay, TRUE, width, height);
		break;
		case GST_MEDIA_PLAY_NORMAL:
			if (mplay->display_mode == GST_MEDIA_PLAY_FULLSCREEN)
				gst_media_play_set_fullscreen (mplay, FALSE, width, height);
		break;
		case GST_MEDIA_PLAY_NOCONTROL:
			break;
		default:
			g_warning("unknown mode %d\n", display_mode);
		break;
	}
	mplay->display_mode = display_mode;
}

GstMediaPlayMode
gst_media_play_get_display_mode (GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	return mplay->display_mode;
}

/* =========================================== */
/*                                             */
/*                GstPlay stuff                */
/*                                             */
/* =========================================== */

void
gst_media_play_set_video_scale (GstMediaPlay *mplay, gfloat scale_factor)
{
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	if (mplay->video_widget)
		g_object_set (	G_OBJECT (mplay->video_widget),
						"scale_factor", scale_factor, NULL);
}

gfloat
gst_media_play_get_video_scale (GstMediaPlay *mplay)
{
	gfloat scale_factor = 1.0;
	g_return_if_fail(mplay != NULL);
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	if (mplay->video_widget)
		g_object_get (	G_OBJECT (mplay->video_widget),
						"scale_factor", &scale_factor, NULL);
	return scale_factor;
}

void
gst_media_play_set_state (GstMediaPlay *mplay, GstElementState state)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	g_return_if_fail(mplay->location != NULL);
	gst_play_set_state (mplay->play, state);
}

GstElementState
gst_media_play_get_state (GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	return gst_play_get_state (mplay->play);
}

void
gst_media_play_set_location (GstMediaPlay *mplay, const gchar *location)
{
	GnomeVFSFileInfo info;
	GstElement *datasrc = NULL;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	g_return_if_fail(location != NULL);

	mplay->location_short = NULL;
	mplay->location = location;
	
	gst_video_widget_set_logo_focus (	GST_VIDEO_WIDGET(mplay->video_widget),
										TRUE);
	gst_play_need_new_video_window ( mplay->play);
	
	/* get a short filename if we can */
	if (GNOME_VFS_OK != gnome_vfs_get_file_info(	location,
													&info,
													GNOME_VFS_FILE_INFO_DEFAULT)
		)
		mplay->location_short = location;
	else
		mplay->location_short = info.name;
	
	g_signal_emit (	mplay,
					gst_media_play_signals [CURRENT_LOCATION],
					0, mplay->location, mplay->location_short);

	gst_media_play_set_state (mplay, GST_STATE_READY);

	if (g_file_exists (location)) {
		g_message ("location is a local file using filesrc");
		datasrc = gst_element_factory_make ("filesrc", "source");
	}
	else {
		g_message ("location is not a local file using gnomevfssrc");
		datasrc = gst_element_factory_make ("gnomevfssrc", "source");
	}
	
	if (GST_IS_ELEMENT(datasrc))
		gst_play_set_data_src (mplay->play, datasrc);
	
	gst_play_set_location (mplay->play, mplay->location);
}

const gchar*
gst_media_play_get_location (GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	return mplay->location;
}

void
gst_media_play_set_volume (GstMediaPlay *mplay, gdouble value)
{
	GtkAdjustment *adj;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	adj = gst_control_get_volume_adjustment(GST_CONTROL(mplay->control));
	
	value = value > 1.0 ? 1.0 : value;
	value = value < 0.0 ? 0.0 : value;
	gtk_adjustment_set_value(adj, value);
}

gdouble
gst_media_play_get_volume (GstMediaPlay *mplay)
{
	GtkAdjustment *adj;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	adj = gst_control_get_volume_adjustment(GST_CONTROL(mplay->control));
	return gtk_adjustment_get_value(adj);
}

/* =========================================== */
/*                                             */
/*                  Tool Box                   */
/*                                             */
/* =========================================== */

gboolean
gst_media_gconf_get_boolean (const gchar *key, gboolean *value)
{
  gchar *full_key;
  GError *error = NULL;

  static GConfClient *_gst_media_gconf_client = NULL;

  if (!_gst_media_gconf_client)
    _gst_media_gconf_client = gconf_client_get_default ();

  full_key = g_strdup_printf ("%s/%s", GCONF_PREFIX, key);
  *value = gconf_client_get_bool (_gst_media_gconf_client, full_key, &error);
  g_free (full_key);
  if (error)
  {
    g_warning ("GConf: %s\n", error->message);
    g_error_free (error);
    return FALSE;
  }
  return TRUE;
}

gint
gst_media_play_handle_keypress (	GtkWidget *widget,
									GdkEventKey *event, 
		                 			GstMediaPlay *mplay)
{
	guint state;
	gboolean handled = FALSE;
  
	g_return_val_if_fail (GST_IS_MEDIA_PLAY (mplay), 0);

	state = event->state;
  
	switch (event->keyval) {
		case GDK_l:
			if (gst_video_widget_get_logo_focus(
						GST_VIDEO_WIDGET(mplay->video_widget)))
				gst_video_widget_set_logo_focus(
								GST_VIDEO_WIDGET(mplay->video_widget),
								FALSE);
			else
				gst_video_widget_set_logo_focus(
								GST_VIDEO_WIDGET(mplay->video_widget),
								TRUE);
		break;
		case GDK_Up:
			gst_media_play_set_volume (mplay, 
					    gst_media_play_get_volume (mplay) + 0.1);
			handled = TRUE;
		break;
		case GDK_Down:
			gst_media_play_set_volume (mplay, 
			    gst_media_play_get_volume (mplay) - 0.1);
			handled = TRUE;
		break;
		case GDK_space:
		case GDK_p:
			if (gst_media_play_get_state (mplay) == GST_STATE_PLAYING) {
				gst_media_play_set_state (mplay, GST_STATE_PAUSED);
			}
			else {
				gst_media_play_set_state (mplay, GST_STATE_PLAYING);
			}
			handled = TRUE;
		break;
		case GDK_Return:
		case GDK_Escape:
			if (mplay->display_mode == GST_MEDIA_PLAY_FULLSCREEN){
				gst_media_play_set_display_mode (	mplay,
													GST_MEDIA_PLAY_NORMAL,
													0,0);
			}
			handled = TRUE;
		break;
	}
  
	return handled;
}

/* =========================================== */
/*                                             */
/*          Widget typing & Creation           */
/*                                             */
/* =========================================== */

GType
gst_media_play_get_type (void)
{
	static GType play_type = 0;
  
	if (!play_type)
	{
		static const GTypeInfo style_info =
			{
				sizeof (GstMediaPlayClass),
				(GBaseInitFunc) NULL,
				(GBaseFinalizeFunc) NULL,
				(GClassInitFunc) gst_media_play_class_init,
				NULL,           /* class_finalize */
				NULL,           /* class_data */
				sizeof (GstMediaPlay),
				0,              /* n_preallocs */
				(GInstanceInitFunc) gst_media_play_init,
			};
		
		play_type = g_type_register_static (GTK_TYPE_VBOX,
						    "GstMediaPlay",
						    &style_info, 0);
	}
	
	return play_type;
}

GstMediaPlay*
gst_media_play_new (void)
{
	return GST_MEDIA_PLAY (g_object_new (GST_TYPE_MEDIA_PLAY, NULL));
}

/* ============================================================= */
/*                                                               */
/*                    Unused for the moment                      */
/*                                                               */
/* ============================================================= */

void
gst_media_play_show_media_info (GstMediaPlay *mplay)
{
	GtkWidget *media_info;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	if (mplay->location != NULL) {
		media_info = gst_media_info_new (mplay->location);
	}
	else {
		media_info = gst_media_info_new ("no file loaded");
	}

	gtk_widget_show (media_info);
}

void
gst_media_play_show_preferences (GstMediaPlay *mplay)
{
	GtkWidget *preferences;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	preferences = gst_preferences_new ();
	
	gtk_widget_show (preferences);
}
